-- (c) 2006 Witold Bolt (ja@hope.art.pl)

--
-- ochrona prosb o wypozyczenie
--

CREATE RULE prosba_insert
AS ON INSERT TO prosba_o_wypozyczenie
-- czy dany uzytkownik bedzie ma prawo wypozyczac ksiazke?
WHERE NEW.uid NOT IN (SELECT * FROM getEgzemplarzAllowedUsers(NEW.eid))
-- czy dany uzytkownik chce wypozyczyc sam od siebie ?
OR (NEW.uid IN (SELECT uid FROM egzemplarz WHERE id=NEW.eid))
DO INSTEAD NOTHING;

CREATE RULE prosba_update
AS ON UPDATE TO prosba_o_wypozyczenie
-- j/w - ochrona przed sytuacja gdy najpierw dodajemy wpis z jakims
-- uzytkownikiem ktory jest uprawniony a potem zmieniamy go na innego
-- ktory juz nie jest
WHERE NEW.uid NOT IN (SELECT * FROM getEgzemplarzAllowedUsers(NEW.eid))
DO INSTEAD NOTHING;

--
-- ochrona historii
--

-- reguly uaktualniania historii
CREATE RULE historia_update
AS ON UPDATE TO historia
-- update odbywa sie w momencie zwrotu
WHERE (OLD.data_zwrotu IS NOT NULL)
-- czy ktos nie oddal ksiazki przed wypozyczeniem ?
    OR (NEW.data_zwrotu < OLD.data_wypozyczenia)
-- czy ktos nie kombinuje ze zmiana inne parametru
    OR (OLD.data_wypozyczenia <> NEW.data_wypozyczenia)
    OR (OLD.uid <> NEW.uid)
    OR (OLD.eid <> NEW.eid)
    OR (OLD.id <> NEW.id)
DO INSTEAD NOTHING;

-- nie wolno wywalac nic z historii
--CREATE RULE historia_delete
--AS ON DELETE TO historia
--DO INSTEAD NOTHING;

--
-- ochrona widoku wypozyczenie przed niedozwolonymi operacjami
--

CREATE RULE wypozyczenie_del
AS ON DELETE TO wypozyczenie
DO INSTEAD NOTHING;

CREATE RULE wypozyczenie_ins
AS ON INSERT TO wypozyczenie
DO INSTEAD NOTHING;

CREATE RULE wypozyczenie_upd
AS ON UPDATE TO wypozyczenie
DO INSTEAD NOTHING;

--
-- ochrona opinii
--

-- egzemplarz

CREATE RULE egzemplarz_opinia_insert
AS ON INSERT TO egzemplarz_opinia
-- nie mozna powiazac jednej opinii z ksiazka i egzemplarzem na raz
WHERE NEW.coid IN (SELECT coid from ksiazka_opinia)
-- nie mozna dwa razy przypisac tej samej opinii
OR NEW.coid IN (SELECT coid from egzemplarz_opinia)
DO INSTEAD NOTHING;

CREATE RULE egzemplarz_opinia_update
AS ON UPDATE TO egzemplarz_opinia
DO INSTEAD NOTHING;

-- ksiazka

CREATE RULE ksiazka_opinia_insert
AS ON INSERT TO ksiazka_opinia
-- nie mozna powiazac jednej opinii z ksiazka i egzemplarzem na raz
WHERE NEW.coid IN (SELECT coid from egzemplarz_opinia)
-- nie mozna dwa razy przypisac tej samej opinii
OR NEW.coid IN (SELECT coid from ksiazka_opinia)
DO INSTEAD NOTHING;

CREATE RULE ksiazka_opinia_update
AS ON UPDATE TO ksiazka_opinia
DO INSTEAD NOTHING;
