-- (c) 2006 Witold Bolt (ja@hope.art.pl)

-- grupa uzytkownikow
CREATE TABLE grupa
(
    id      serial NOT NULL PRIMARY KEY,
    nazwa   varchar(32)
);

-- uzytkownik systemu
CREATE TABLE uzytkownik
(
    -- dane obowiazkowe
    id                          serial          NOT NULL PRIMARY KEY,
    login                       varchar(32)     NOT NULL,
    passwd                      varchar(32)     NOT NULL,
    e_mail                      varchar(64)     NOT NULL,

    -- dane nieobowiazkowe
    imie_nazwisko               varchar(40),
    miasto                      varchar(32)
);

-- tabela mowi o przynaleznosci uzytkownikow do grup
CREATE TABLE uzytkownik_grupa
(
    uid integer  NOT NULL,
    gid integer  NOT NULL,

    CONSTRAINT gid_fk   FOREIGN KEY (gid) REFERENCES grupa(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT uid_fk   FOREIGN KEY (uid) REFERENCES uzytkownik(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

-- kategorie do ktorej naleza ksiazki
CREATE TABLE kategoria
(
    id      serial          NOT NULL PRIMARY KEY,
    nazwa   varchar(128)    NOT NULL,

    pid     integer,

    CONSTRAINT  pid_fk FOREIGN KEY (pid) REFERENCES kategoria(id)
                    ON UPDATE CASCADE
);

-- opinia ktora moze byc przypisana zarowno do ksiazki jak i egzemplarza
CREATE TABLE opinia
(
    id              serial  NOT NULL    PRIMARY KEY,
    uid             integer NOT NULL,
    tresc           text    NOT NULL,
    data_wpisania   date    NOT NULL,

    CONSTRAINT uid_fk   FOREIGN KEY (uid) REFERENCES uzytkownik(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

-- ksiazka
CREATE TABLE ksiazka
(
    id      serial      NOT NULL    PRIMARY KEY,
    autor   varchar(512) NOT NULL,
    tytul   varchar(512) NOT NULL
);

-- egzemplarz danej ksiazki
CREATE TABLE egzemplarz
(
    -- pola obowiazkowe
    id                      serial  NOT NULL    PRIMARY KEY,
    kid                     integer NOT NULL,
    uid                     integer NOT NULL,
    moze_byc_wypozyczony   boolean NOT NULL DEFAULT TRUE,

    -- pola opcjonalne
    liczba_stron    integer,
    rok_wyd         numeric(4),
    miasto_wyd      varchar(32),
    wydawnictwo     varchar(32),
    wydanie         numeric(4),

    CONSTRAINT uid_fk   FOREIGN KEY (uid) REFERENCES uzytkownik(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT kid_fk   FOREIGN KEY (kid) REFERENCES ksiazka(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

-- historia wypozyczen
CREATE TABLE historia
(
    id                  serial NOT NULL PRIMARY KEY,
    uid                 integer NOT NULL,
    eid                 integer NOT NULL,
    data_wypozyczenia   date    NOT NULL,
    data_zwrotu         date,

    CONSTRAINT uid_fk   FOREIGN KEY (uid) REFERENCES uzytkownik(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT eid_fk   FOREIGN KEY (eid) REFERENCES egzemplarz(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE kategoria_ksiazka
(
    kid integer  NOT NULL,
    cid integer  NOT NULL,

    CONSTRAINT cid_fk   FOREIGN KEY (cid) REFERENCES kategoria(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT kid_fk   FOREIGN KEY (kid) REFERENCES ksiazka(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE egzemplarz_opinia
(
    eid integer   NOT NULL,
    coid integer  NOT NULL,

    CONSTRAINT coid_fk   FOREIGN KEY (coid) REFERENCES opinia(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT eid_fk   FOREIGN KEY (eid) REFERENCES egzemplarz(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE ksiazka_opinia
(
    kid integer   NOT NULL,
    coid integer  NOT NULL,

    CONSTRAINT coid_fk   FOREIGN KEY (coid) REFERENCES opinia(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT kid_fk   FOREIGN KEY (kid) REFERENCES ksiazka(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE prosba_o_wypozyczenie
(
    -- kto prosi:
    uid         integer  NOT NULL,

    -- o co prosi:
    eid         integer  NOT NULL,

    -- data prosby:
    data        timestamp     NOT NULL,

    CONSTRAINT uid_fk FOREIGN KEY (uid) REFERENCES uzytkownik(id)
                        ON UPDATE CASCADE ON DELETE CASCADE,

    CONSTRAINT eid_fk FOREIGN KEY (eid) REFERENCES egzemplarz(id)
                        ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE VIEW wypozyczenie AS
    SELECT uid,eid,data_wypozyczenia
        FROM historia WHERE data_zwrotu IS NULL;
